"use strict";
const command_1 = require("@oclif/command");
const SilverstripeOrgFilter_1 = require("./Filters/SilverstripeOrgFilter");
const SupportedOrgFilter_1 = require("./Filters/SupportedOrgFilter");
const TravisClient_1 = require("./TravisClient");
class FetchTravisData extends command_1.Command {
    constructor(argv, config) {
        super(argv, config);
        this.buildURL = this.buildURL.bind(this);
        this.print = this.print.bind(this);
        this.getBuildsForOrgs = this.getBuildsForOrgs.bind(this);
        this.getBuildsForOrg = this.getBuildsForOrg.bind(this);
        this.printBranches = this.printBranches.bind(this);
        this.toJson = this.toJson.bind(this);
    }
    getOrgs() {
        const filters = [
            'dnadesign',
            'bringyourownideas',
            'colymba',
            'hafriedlander',
            'lekoala',
            'silverstripe-themes',
            'symbiote',
            'tijsverkoyen',
            'tractorcow-farm',
            'tractorcow',
            'undefinedoffset',
        ].map(org => new SupportedOrgFilter_1.SupportedOrgFilter(org));
        return filters.concat([SilverstripeOrgFilter_1.SilverstripeOrgFilter]);
    }
    async run() {
        const { flags: { token, domain, output, failed, verbose } } = this.parse(FetchTravisData);
        const travisToken = token !== null && token !== void 0 ? token : process.env.TRAVIS_TOKEN;
        if (!travisToken) {
            this.error('You must provide a travis token with `--token` or with TRAVIS_TOKEN environement variable.');
            return;
        }
        const lcDomain = domain.toLocaleLowerCase();
        if (lcDomain !== 'org' && lcDomain !== 'com') {
            this.error('`domain` must be either `com` or `org`.');
            return;
        }
        this.client = new TravisClient_1.TravisClient(travisToken, lcDomain, verbose ? this.log : () => { });
        const outputMethod = output === 'pretty' ? this.printBranches :
            output === 'json' ? this.toJson :
                () => { };
        this.getBuildsForOrgs(this.getOrgs())
            .then(failed ? this.filterFailed : b => b)
            .then(outputMethod);
    }
    /**
     * Filter out failed branches
     * @param branches
     */
    filterFailed(branches) {
        return branches.filter(branch => {
            var _a;
            return (branch.last_build &&
                ['failed', 'errored'].indexOf((_a = branch.last_build) === null || _a === void 0 ? void 0 : _a.state) !== -1);
        });
    }
    toJson(branches) {
        let results = {};
        branches.forEach(({ last_build, repository: { slug }, name }) => {
            if (!last_build) {
                return;
            }
            if (!results[slug]) {
                results[slug] = {};
            }
            const { state, started_at, id } = last_build;
            results[slug][name] = { state, started_at, id };
        });
        this.log(JSON.stringify(results));
    }
    printBranches(branches) {
        branches.forEach(this.print);
    }
    print(branch) {
        var _a, _b;
        let stateColour;
        switch ((_a = branch.last_build) === null || _a === void 0 ? void 0 : _a.state) {
            case "failed":
                stateColour = `\x1b[41m`;
                break;
            case "passed":
                stateColour = `\x1b[42m`;
                break;
            case "running":
                stateColour = `\x1b[43m`;
                break;
            case "errored":
                stateColour = `\x1b[45m`;
                break;
            default:
                stateColour = `\x1b[46m`;
        }
        const url = branch.last_build ? this.buildURL(branch.repository.slug, branch.last_build.id) : '';
        this.log(`\x1b[36m${branch.repository.slug}\x1b[0m:\x1b[35m${branch.name}\x1b[0m \t${stateColour} ${(_b = branch.last_build) === null || _b === void 0 ? void 0 : _b.state} \x1b[0m \t${url}`);
    }
    getBuildsForOrgs(orgs) {
        return Promise.all(orgs.map(this.getBuildsForOrg)).then((branches) => [].concat(...branches));
    }
    getBuildsForOrg(org) {
        if (!this.client) {
            this.error('Client has not been initialsed');
        }
        const client = this.client;
        return client.getRepos(org.name())
            .then((repos) => repos ? repos.filter(org.includeRepo) : [])
            .then((repos) => Promise.all(repos.map(({ slug }) => client.getBranches(slug))))
            .then((branches) => [].concat(...branches))
            .then((branches) => branches.filter(org.includeBranch));
    }
    buildURL(slug, id) {
        return `https://travis-ci.org/github/${slug}/builds/${id}`;
    }
}
FetchTravisData.description = 'Fetch build status for silverstripe module';
FetchTravisData.flags = {
    // add --version flag to show CLI version
    version: command_1.flags.version({ char: 'v' }),
    help: command_1.flags.help({ char: 'h' }),
    // flag with a value (-n, --name=VALUE)
    token: command_1.flags.string({ char: 't', description: 'Travis token' }),
    domain: command_1.flags.string({ char: 'd', description: 'Travis domain to target: org or com', default: 'com' }),
    output: command_1.flags.enum({ char: 'o', options: ['pretty', 'json'], default: 'pretty', description: 'Control the output format' }),
    failed: command_1.flags.boolean({ char: 'f', default: false, description: 'Only display failed build.' }),
    verbose: command_1.flags.boolean({ default: false, description: 'Print out debug statement.' })
};
FetchTravisData.args = [{ name: 'command' }];
module.exports = FetchTravisData;
